@@ -1,11 +1,12 @@
 module HTTP
   class Response
     class Streamer
-      def initialize(str)
+      def initialize(str, encoding: Encoding::BINARY)
         @io = StringIO.new str
+        @encoding = encoding
       end
 
-      def readpartial(size = nil)
+      def readpartial(size = nil, outbuf = nil)
         unless size
           if defined?(HTTP::Client::BUFFER_SIZE)
             size = HTTP::Client::BUFFER_SIZE
@@ -14,7 +15,8 @@ module HTTP
           end
         end
 
-        @io.read size
+        chunk = @io.read size, outbuf
+        chunk.force_encoding(@encoding) if chunk
       end
 
       def close
