@@ -1,7 +1,8 @@
 module HTTP
   class WebMockPerform
-    def initialize(request, &perform)
+    def initialize(request, options, &perform)
       @request = request
+      @options = options
       @perform = perform
       @request_signature = nil
     end
@@ -38,12 +39,15 @@ module HTTP
       webmock_response.raise_error_if_any
 
       invoke_callbacks(webmock_response, real_request: false)
-      ::HTTP::Response.from_webmock webmock_response, request_signature
+      response = ::HTTP::Response.from_webmock @request, webmock_response, request_signature
+
+      @options.features.each { |_name, feature| response = feature.wrap_response(response) }
+      response
     end
 
     def raise_timeout_error
       raise Errno::ETIMEDOUT if HTTP::VERSION < "1.0.0"
-      raise HTTP::ConnectionError, "connection error: #{Errno::ETIMEDOUT.new}"
+      raise HTTP::TimeoutError, "connection error: #{Errno::ETIMEDOUT.new}"
     end
 
     def perform
