@@ -16,6 +16,21 @@ module WebMock
       self
     end
 
+    def at_least_once
+      @request_execution_verifier.at_least_times_executed = 1
+      self
+    end
+
+    def at_least_twice
+      @request_execution_verifier.at_least_times_executed = 2
+      self
+    end
+
+    def at_least_times(times)
+      @request_execution_verifier.at_least_times_executed = times
+      self
+    end
+
     def with(options = {}, &block)
       @request_execution_verifier.request_pattern.with(options, &block)
       self
