@@ -28,8 +28,8 @@ module PatronSpecHelper
       end
     end
 
-    status_line_pattern = %r(\AHTTP/(\d+\.\d+)\s+(\d\d\d)\s*([^\r\n]+)?)
-    message = response.status_line.match(status_line_pattern)[3] || ""
+    status_line_pattern = %r(\AHTTP/(\d+(\.\d+)?)\s+(\d\d\d)\s*([^\r\n]+)?)
+    message = response.status_line.match(status_line_pattern)[4] || ""
 
     OpenStruct.new({
       body: response.body,
